<?php

namespace App\Jobs;

use App\Models\RechargeApi;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Log;

class UpdateChargeDiscountJob implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $rechargeObj;
    private $addressID;
    private $discountCode;
    private $productID;
    private $variantID;
    private $discountPrice;
    private $requestData;
    
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($requestData) {

        $this->requestData = $requestData;
        $this->addressID = !empty($this->requestData->address_id) ? trim($this->requestData->address_id) : "";
        $this->discountCode = !empty($this->requestData->discount_codes[0]->code) ? 
        trim($this->requestData->discount_codes[0]->code) : "";
        $this->productID = !empty($this->requestData->line_items[0]->shopify_product_id) ? 
        trim($this->requestData->line_items[0]->shopify_product_id) : "";
        $this->variantID = !empty($this->requestData->line_items[0]->shopify_variant_id) ? 
        trim($this->requestData->line_items[0]->shopify_variant_id) : "";
        $this->discountPrice = !empty($this->requestData->discount_codes[0]->amount) ? 
        trim($this->requestData->discount_codes[0]->amount) : "";
    }

    public function handle() {

        $this->rechargeObj = new RechargeApi();

        if ($this->addressID && $this->discountCode) {

            Log::channel('chargeDiscountApplyLog')->info("Address Id: ".$this->addressID);
            Log::channel('chargeDiscountApplyLog')->info("Discount Code: ".$this->discountCode);
            Log::channel('chargeDiscountApplyLog')->info("Discount Price: ".$this->discountPrice);

            if (strpos($this->discountCode, "-applied") === false) {

                if ($this->discountPrice == 0) {

                    $valid = $this->checkDiscountCodeValidation($this->discountCode);
                    if ($valid) {
                        Log::channel('chargeDiscountApplyLog')->info("Discount code validation success");
                        $chargeDetails = $this->rechargeObj->getChargeByAddressID($this->addressID);
                        
                        Log::channel('chargeDiscountApplyLog')->info("Charge details - ".json_encode($chargeDetails));

                        if (isset($chargeDetails->error) || isset($chargeDetails->errors) ||
                            empty($chargeDetails) || empty($chargeDetails->charges)) {
                            
                            $status = "failed";
                            $error = "Unknown error";

                            if (isset($chargeDetails->error)) {
                                $error = json_encode($chargeDetails->error);
                            } elseif (isset($chargeDetails->errors)) {
                                $error = json_encode($chargeDetails->errors);
                            } else {
                                $error = "The request is not correct or missing the required parameters";
                            }

                            Log::channel('chargeDiscountApplyLog')->info("Failed to get the charge details for AddressID (" .
                            $this->addressID .", Error:".$error.")");
                            die();

                        } else {
                            $chargeID = '';
                            foreach ($chargeDetails->charges as $charge) {

                                $chargeProductId = $charge->line_items[0]->shopify_product_id;
                                $chargeVariantId = $charge->line_items[0]->shopify_variant_id;

                                if (trim($charge->address_id) == trim($this->addressID) && trim($chargeProductId) == trim($this->productID)
                                    || trim($chargeVariantId) == trim($this->variantID) && trim(strtolower($charge->status)) == "queued") {

                                    $chargeID = $charge->id;
                                    break;
                                }
                                continue;
                            }

                            if ($chargeID) {
                                Log::channel('chargeDiscountApplyLog')->info("Charge Id - ". $chargeID);

                                $addressDetails = $this->rechargeObj->getAddressByAddressID($this->addressID);
                                
                                if (isset($addressDetails->error) || isset($addressDetails->errors) ||
                                    empty($addressDetails) || empty($addressDetails->address)) {
                                    
                                    $status = "failed";
                                    $error = "Unknown error";

                                    if (isset($addressDetails->error)) {
                                        $error = json_encode($addressDetails->error);
                                    } elseif (isset($addressDetails->errors)) {
                                        $error = json_encode($addressDetails->errors);
                                    } else {
                                        $error = "The request is not correct or missing the required parameters";
                                    }

                                    Log::channel('chargeDiscountApplyLog')->info("Failed to get the address details for AddressID (" .
                                    $this->addressID . "), Error:".$error.")");
                                    die();

                                } else {
                                    Log::channel('chargeDiscountApplyLog')->info("Address Data - ". json_encode($addressDetails));
                                    
                                    if ($addressDetails->address->discount_id != null) {
                                        Log::channel('chargeDiscountApplyLog')->info("Discount id is not null in the address");
                                        Log::channel('chargeDiscountApplyLog')->info("Address discount id - ".$addressDetails->address->discount_id);
                                        
                                        $addressDiscountId = trim($addressDetails->address->discount_id);
                                        
                                        $addrDiscountRemoveResponse = $this->rechargeObj->removeDiscountFromAddress($this->addressID);
                                        Log::channel('chargeDiscountApplyLog')->info("Discount removing response - ".json_encode($addrDiscountRemoveResponse));

                                        if (isset($addrDiscountRemoveResponse->error) || isset($addrDiscountRemoveResponse->errors) ||
                                            empty($addrDiscountRemoveResponse) || empty($addrDiscountRemoveResponse->address)) {
                                            
                                            $status = "failed";
                                            $error = "Unknown error";

                                            if (isset($addrDiscountRemoveResponse->error)) {
                                                $error = json_encode($addrDiscountRemoveResponse->error);
                                            } elseif (isset($addrDiscountRemoveResponse->errors)) {
                                                $error = json_encode($addrDiscountRemoveResponse->errors);
                                            } else {
                                                $error = "The request is not correct or missing the required parameters";
                                            }

                                            Log::channel('chargeDiscountApplyLog')->info("Failed to remove the discount from the AddressID (" .
                                            $this->addressID . "), Error:".$error.")");
                                            die();

                                        } else {
                                            Log::channel('chargeDiscountApplyLog')->info("Successfully removed the discount from the address - ".json_encode($addrDiscountRemoveResponse));
                                        }

                                        $discountDetails =  $this->rechargeObj->getDiscountDetailsByID($addressDiscountId);
                                        Log::channel('chargeDiscountApplyLog')->info("Discount data - ".json_encode($discountDetails));

                                        if (isset($discountDetails->error) || isset($discountDetails->errors) ||
                                            empty($discountDetails) || empty($discountDetails->discount)) {
                                            
                                            $status = "failed";
                                            $error = "Unknown error";

                                            if (isset($discountDetails->error)) {
                                                $error = json_encode($discountDetails->error);
                                            } elseif (isset($discountDetails->errors)) {
                                                $error = json_encode($discountDetails->errors);
                                            } else {
                                                $error = "The request is not correct or missing the required parameters";
                                            }

                                            Log::channel('chargeDiscountApplyLog')->info("Failed to fetch the discount details of discountId - (" .
                                            $addressDiscountId . "), Error:".$error.")");
                                            // die();

                                        } else {
                                            Log::channel('chargeDiscountApplyLog')->info("Discount data is fetched successfully");

                                            $deactivateRequest = array(
                                                "status" => "fully_disabled"
                                            ); 
                                            Log::channel('chargeDiscountApplyLog')->info("Deactivate request - ".json_encode($deactivateRequest));
                                            $deactivateResponse = $this->rechargeObj->updateDiscount($addressDiscountId, $deactivateRequest);

                                            if (isset($deactivateResponse->error) || isset($deactivateResponse->errors) || empty($deactivateResponse)
                                                 || empty($deactivateResponse->discount) || trim($deactivateResponse->discount->status) != "fully_disabled") {
                                                
                                                $status = "failed";
                                                $error = "Unknown error";

                                                if (isset($deactivateResponse->error)) {
                                                    $error = json_encode($deactivateResponse->error);
                                                } elseif (isset($deactivateResponse->errors)) {
                                                    $error = json_encode($deactivateResponse->errors);
                                                } else {
                                                    $error = "The request is not correct or missing the required parameters";
                                                }

                                                Log::channel('chargeDiscountApplyLog')->info("Failed to deactivate the discount - (" .
                                                $addressDiscountId . "), Error:".$error.")");

                                            } else {
                                                Log::channel('chargeDiscountApplyLog')->info("Successfully deactivated the discount code");
                                            }
                                        }

                                    } else {
                                        Log::channel('chargeDiscountApplyLog')->info("Discount id is null in the address.");
                                    }
                                }

                                $discountRequest = array(
                                    "discount_code" => $this->discountCode."-applied",
                                ); 

                                Log::channel('chargeDiscountApplyLog')->info("Discount request - ". json_encode($discountRequest));

                                $discountResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($chargeID, json_encode($discountRequest));

                                Log::channel('chargeDiscountApplyLog')->info("Discount response - ". json_encode($discountResponse));
                            
                                if (isset($discountResponse->error) || isset($discountResponse->errors) ||
                                    empty($discountResponse) || empty($discountResponse->charge)) {
                                    Log::channel('chargeDiscountApplyLog')->info("Failed to apply the discount to the charge - " .$chargeID . ":- ". json_encode($discountResponse));
                                } else {
                                    Log::channel('chargeDiscountApplyLog')->info("Successfully applied the discount to the charge - " .$chargeID . ":- ". json_encode($discountResponse));
                                }
                            } else {
                                Log::channel('chargeDiscountApplyLog')->info("No queued charge in recharge");
                            }
                        }
                    } else {
                        Log::channel('chargeDiscountApplyLog')->info("The discount code is not valid");
                    }
                } else {
                    Log::channel('chargeDiscountApplyLog')->info("The discount price is greater than zero");
                }
            } else {
                Log::channel('chargeDiscountApplyLog')->info("The discount code has the suffix '-applied'");
            }
        }
    }

    /**
     * Checking discount code validation
     */
    public function checkDiscountCodeValidation($discountCode) {

        $validation = false;
        if ($discountCode) {
            $discArray = explode('-', $discountCode);
            //echo '<br>'.count($discArray);
            if (count($discArray) == 3) {
                if (strlen($discArray[0]) == 2 && strlen($discArray[1]) == 2 && strlen($discArray[2]) == 5) {
                    if (ctype_upper($discArray[0]) && ctype_upper($discArray[1]) && ctype_upper($discArray[2][0])) {
                        if ($discArray[2][0] == 'A' || $discArray[2][0] == 'B' || $discArray[2][0] == 'C') {
                            $validation = true;
                        }
                    }
                }
            }
        }
        return $validation;
    }
}
